@@ -176,11 +176,10 @@ class EvacuateInstancesTask(base.MasakariTask):
                 periodic_call_stopped.wait)
         except etimeout.Timeout:
             with excutils.save_and_reraise_exception():
-                periodic_call_stopped.stop()
                 msg = ("Instance '%(uuid)s' is successfully evacuated but "
-                       "failed to stop.") % {'uuid': instance.id}
+                       "timeout to stop.") % {'uuid': instance.id}
                 LOG.warning(msg)
-        else:
+        finally:
             periodic_call_stopped.stop()
 
     def _evacuate_and_confirm(self, context, instance, host_name,
@@ -200,6 +199,12 @@ class EvacuateInstancesTask(base.MasakariTask):
         def _wait_for_evacuation_confirmation():
             old_vm_state, new_vm_state, instance_host = (
                 self._get_state_and_host_of_instance(context, instance))
+
+            if (new_vm_state == 'error' and
+                    new_vm_state != old_vm_state):
+                raise exception.InstanceEvacuateFailed(
+                    instance_uuid=instance.id)
+
             if instance_host != host_name:
                 if ((old_vm_state == 'error' and
                     new_vm_state == 'active') or
@@ -217,9 +222,11 @@ class EvacuateInstancesTask(base.MasakariTask):
                     CONF.wait_period_after_evacuation,
                     periodic_call.wait)
             except etimeout.Timeout:
-                # Instance is not evacuated in the expected time_limit.
-                failed_evacuation_instances.append(instance.id)
-            else:
+                with excutils.save_and_reraise_exception():
+                    msg = ("Timeout for instance '%(uuid)s' evacuation."
+                           % {'uuid': instance.id})
+                    LOG.warning(msg)
+            finally:
                 # stop the periodic call, in case of exceptions or
                 # Timeout.
                 periodic_call.stop()
@@ -240,6 +247,12 @@ class EvacuateInstancesTask(base.MasakariTask):
                 if vm_state == 'resized' and power_state != SHUTDOWN:
                     stop_instance = False
 
+            elif vm_state == 'stopped' and task_state is None:
+                # If vm_state is stopped and task_state is none, the instance
+                # will be recovered with vm_state 'stopped'.
+                # So it doesn't need to stop the instance after evacuation.
+                stop_instance = False
+
             elif task_state is not None:
                 # Nova fails evacuation when the instance's task_state is not
                 # none. In this case, masakari resets the instance's vm_state
@@ -266,9 +279,10 @@ class EvacuateInstancesTask(base.MasakariTask):
         except etimeout.Timeout:
             # Instance is not stop in the expected time_limit.
             failed_evacuation_instances.append(instance.id)
-        except Exception:
+        except Exception as e:
             # Exception is raised while resetting instance state or
             # evacuating the instance itself.
+            LOG.warning(str(e))
             failed_evacuation_instances.append(instance.id)
         finally:
             if not instance_already_locked:
